/* -*- C++ -*- */
// LogMgr_i.h,v 1.1 1999/09/15 22:23:26 pradeep Exp

// ============================================================================
// = LIBRARY
//   orbsvcs
//
// = FILENAME
//   LogMgr_i.h
//
// = DESCRIPTION
//   Implementation of the LogManager i,e DsLogAdmin::LogMgr interface.
//
// = AUTHOR
//    Matthew Braun (mjb2@cs.wustl.edu) and Pradeep Gore <pradeep@cs.wustl.edu>
//
// ============================================================================

#ifndef TLS_LOGMGR_I_H
#define TLS_LOGMGR_I_H

#include "orbsvcs/DsLogAdminS.h"
#include "orbsvcs/Log/BasicLog_i.h"

class LogMgr_i : public virtual POA_DsLogAdmin::LogMgr
{
  // = TITLE
  //   LogMgr_i
  // = DESCRIPTION
  //   This class has methods to query for Log objects.
  //
public:
  // Initialization and Termination Methods
  LogMgr_i (void);
  // Constructor.

  ~LogMgr_i ();
  // Destructor.

  DsLogAdmin::LogList *
    list_logs (CORBA::Environment &ACE_TRY_ENV =
               CORBA::default_environment ())
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  // Lists all logs created by the log factory

  DsLogAdmin::Log_ptr
    find_log (DsLogAdmin::LogId id,
              CORBA::Environment &ACE_TRY_ENV =
              CORBA::default_environment ())
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  // Returns a reference to the log with the supplied id

  DsLogAdmin::LogIdList *
  list_logs_by_id (CORBA::Environment &ACE_TRY_ENV =
		   CORBA::default_environment ())
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  // Lists all log ids

 int remove (DsLogAdmin::LogId id);
 // remove the given entry from the hash table.

protected:
  typedef ACE_Hash_Map_Manager <DsLogAdmin::LogId,
    DsLogAdmin::BasicLog_var,ACE_SYNCH_MUTEX> HASHMAP;
  // Define the HASHMAP.

  HASHMAP hash_map_;
  // The map of Logs created.

  DsLogAdmin::LogId max_id_;
  // The Max id assigned so far.
};
#endif /* TLS_LOGMGR_I_H */
